package com.fuyuanshen.modules.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fuyuanshen.modules.system.domain.DeviceTypeGrants;
import com.fuyuanshen.modules.system.mapper.DeviceTypeGrantsMapper;
import com.fuyuanshen.modules.utils.NanoId;
import lombok.RequiredArgsConstructor;
import com.fuyuanshen.modules.security.service.UserCacheManager;
import com.fuyuanshen.modules.security.service.dto.JwtUserDto;
import com.fuyuanshen.modules.system.constant.UserConstants;
import com.fuyuanshen.modules.system.domain.DeviceType;
import com.fuyuanshen.modules.system.domain.User;
import com.fuyuanshen.modules.system.domain.dto.DeviceTypeQueryCriteria;
import com.fuyuanshen.modules.system.mapper.DeviceTypeMapper;
import com.fuyuanshen.modules.system.mapper.UserMapper;
import com.fuyuanshen.modules.system.service.DeviceTypeService;
import com.fuyuanshen.utils.PageResult;
import com.fuyuanshen.utils.PageUtil;
import com.fuyuanshen.utils.RedisUtils;
import com.fuyuanshen.utils.SecurityUtils;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author: WY
 * @Date: 2025/5/14
 **/
@Service
@RequiredArgsConstructor
public class DeviceTypeServiceImpl extends ServiceImpl<DeviceTypeMapper, DeviceType> implements DeviceTypeService {

    private final DeviceTypeMapper deviceTypeMapper;
    private final UserMapper userMapper;
    private final UserCacheManager userCacheManager;
    private final DeviceTypeGrantsMapper deviceTypeGrantsMapper;


    /**
     * 分页查询设备类型
     *
     * @param criteria 条件
     * @param page     分页参数
     * @return
     */
    @Override
    public PageResult<DeviceType> queryAll(DeviceTypeQueryCriteria criteria, Page<Object> page) {
        JwtUserDto userCache = userCacheManager.getUserCache(SecurityUtils.getCurrentUsername());
        User currentUser = userMapper.findByUsername(SecurityUtils.getCurrentUsername());
        // 非超级管理员增加数据隔离
        // if (userCache.getUser().getTenantId() != null && !userCache.getUser().getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
        //     List<User> subUsers = userMapper.findUserTree(userCache.getUser().getId());
        //     Set<Long> subUserIds = subUsers.stream().map(User::getId).collect(Collectors.toSet());
        //     criteria.setCustomerId(subUserIds);
        // }

        // 非超级管理员增加数据隔离(根据租户隔离)
        // if (userCache.getUser().getTenantId() != null && !userCache.getUser().getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
        //     List<User> users = userMapper.findByTenantId(userCache.getUser().getTenantId());
        //          Set<Long> userIds = users.stream().map(User::getId).collect(Collectors.toSet());
        //          criteria.setCustomerId(userIds);
        // 非超级管理员增加数据隔离(只能看到自己创建的设备类型)
        if (currentUser.getTenantId() != null && !currentUser.getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
            //
            QueryWrapper<User> wrapper = new QueryWrapper<>();
            wrapper.eq("username", SecurityUtils.getCurrentUsername());
            User user = userMapper.selectOne(wrapper);
            criteria.setCustomerId(user.getId());
        }

        return PageUtil.toPage(deviceTypeMapper.findAll(criteria, page));
    }


    /**
     * 查询所有设备类型
     *
     * @return
     */
    @Override
    public List<DeviceType> queryDeviceTypes() {
        // JwtUserDto userCache = userCacheManager.getUserCache(SecurityUtils.getCurrentUsername());
        User currentUser = userMapper.findByUsername(SecurityUtils.getCurrentUsername());
        DeviceTypeQueryCriteria criteria = new DeviceTypeQueryCriteria();
        // 非超级管理员增加数据隔离(根据租户隔离)
        // if (userCache.getUser().getTenantId() != null && !userCache.getUser().getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
        //     List<User> users = userMapper.findByTenantId(userCache.getUser().getTenantId());
        //     Set<Long> userIds = users.stream().map(User::getId).collect(Collectors.toSet());
        //     criteria.setCustomerIds(userIds);
        // }

        // 非超级管理员增加数据隔离(只能看到自己创建的设备类型)
        if (currentUser.getTenantId() != null && !currentUser.getTenantId().equals(UserConstants.SUPER_ADMIN_ID)) {
            //
            QueryWrapper<User> wrapper = new QueryWrapper<>();
            wrapper.eq("username", SecurityUtils.getCurrentUsername());
            User user = userMapper.selectOne(wrapper);
            criteria.setCustomerId(user.getId());
        }

        return deviceTypeMapper.findAll(criteria);
    }


    @Override
    public List<DeviceType> queryAll(DeviceTypeQueryCriteria criteria) {
        return deviceTypeMapper.findAll(criteria);
    }


    /**
     * 新增设备类型
     *
     * @param resources /
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void create(DeviceType resources) {
        Long deviceTypeId = NanoId.generate(16);
        resources.setId(deviceTypeId);
        resources.setCustomerId(SecurityUtils.getCurrentUserId());
        deviceTypeMapper.insert(resources);

        // 自动授权给自己
        DeviceTypeGrants deviceTypeGrants = new DeviceTypeGrants();
        deviceTypeGrants.setDeviceTypeId(deviceTypeId);
        deviceTypeGrants.setCustomerId(SecurityUtils.getCurrentUserId());
        deviceTypeGrants.setGrantorCustomerId(SecurityUtils.getCurrentUserId());
        deviceTypeGrants.setGrantedAt(new Date());
        deviceTypeGrantsMapper.insert(deviceTypeGrants);

    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(DeviceType resources) {
        DeviceType deviceType = getById(resources.getId());
        deviceType.copy(resources);
        Timestamp timestamp = new Timestamp(System.currentTimeMillis());
        deviceType.setUpdateTime(timestamp);
        deviceTypeMapper.updateById(deviceType);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteAll(List<Long> ids) {
        deviceTypeMapper.deleteBatchIds(ids);
    }


}
